/**
 * PETALS - PETALS Services Platform. Copyright (c) 2005 EBM Websourcing,
 * http://www.ebmwebsourcing.com/
 * 
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version. This library is distributed in the hope that it will be
 * useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser
 * General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 * 
 * ---------------------------------------------------------------------------
 * ---------------------------------------------------------------------------
 */

package org.ow2.petals.component.framework.notification;

import java.io.File;
import java.net.MalformedURLException;
import java.net.URL;
import java.util.logging.Level;
import java.util.logging.Logger;

import javax.jbi.component.ComponentContext;
import javax.jbi.messaging.MessageExchange;
import javax.jbi.messaging.MessagingException;

import org.apache.commons.lang.NotImplementedException;
import org.ow2.easywsdl.wsdl.api.WSDLException;
import org.ow2.petals.component.framework.api.Constants;
import org.ow2.petals.component.framework.api.exception.PEtALSCDKException;
import org.ow2.petals.component.framework.api.notification.AsyncSender;
import org.ow2.petals.component.framework.api.notification.NotificationBrokerController;
import org.ow2.petals.component.framework.api.notification.NotificationConsumer;
import org.ow2.petals.component.framework.api.notification.NotificationProducer;
import org.ow2.petals.component.framework.api.notification.StatusListener;
import org.ow2.petals.component.framework.api.notification.Transformer;
import org.w3c.dom.Document;

import com.ebmwebsourcing.wsstar.addressing.definition.WSAddressingFactory;
import com.ebmwebsourcing.wsstar.addressing.definition.api.EndpointReferenceType;
import com.ebmwebsourcing.wsstar.addressing.definition.api.ReferenceParametersType;
import com.ebmwebsourcing.wsstar.addressing.definition.api.WSAddressingException;
import com.ebmwebsourcing.wsstar.notification.definition.WSNotificationFactory;
import com.ebmwebsourcing.wsstar.notification.definition.basenotification.api.Message;
import com.ebmwebsourcing.wsstar.notification.definition.basenotification.api.NotificationMessageHolderType;
import com.ebmwebsourcing.wsstar.notification.definition.basenotification.api.Notify;
import com.ebmwebsourcing.wsstar.notification.definition.basenotification.api.SubscribeResponse;
import com.ebmwebsourcing.wsstar.notification.definition.basenotification.api.TopicExpressionType;
import com.ebmwebsourcing.wsstar.notification.definition.basenotification.api.UnsubscribeResponse;
import com.ebmwebsourcing.wsstar.notification.definition.brokerednotification.api.DestroyRegistration;
import com.ebmwebsourcing.wsstar.notification.definition.brokerednotification.api.DestroyRegistrationResponse;
import com.ebmwebsourcing.wsstar.notification.definition.brokerednotification.api.RegisterPublisher;
import com.ebmwebsourcing.wsstar.notification.definition.brokerednotification.api.RegisterPublisherResponse;
import com.ebmwebsourcing.wsstar.notification.definition.inout.WSNotificationWriter;
import com.ebmwebsourcing.wsstar.notification.definition.topics.WstopConstants;
import com.ebmwebsourcing.wsstar.notification.definition.utils.WSNotificationException;
import com.ebmwebsourcing.wsstar.notification.extension.api.ResourcesUuidType;
import com.ebmwebsourcing.wsstar.notification.extension.api.SOAParameterType;
import com.ebmwebsourcing.wsstar.notification.extension.utils.WSNotificationExtensionException;
import com.ebmwebsourcing.wsstar.notification.extension.utils.WsnSpecificTypeHelper;

/**
 * The controller of the broker. Used by the broker to centralize all the
 * notification actors actions.
 * 
 * @author Frederic Gardes, chamerling
 * 
 */
public class ComponentNotificationBrokerController implements NotificationBrokerController {

    /**
     * The transformer of notification
     */
    private Transformer transformer;

    /**
     * The Notification Producer
     */
    private NotificationProducer notificationProducer;

    /**
     * The Notification Consumer
     */
    private NotificationConsumer notificationConsumer;

    /**
     * The writer of document to unmarshal notifications
     */
    private WSNotificationWriter wsnWriter;

    private NotificationContext notificationContext;

    /**
     * The logger
     */
    private Logger logger;

    private NotificationService notificationService;

    public ComponentNotificationBrokerController(final Logger logger,
            final ComponentContext componentContext, NotificationService notificationService)
            throws WSNotificationException, WSDLException, WSAddressingException,
            WSNotificationExtensionException, MessagingException {
        super();
        this.notificationService = notificationService;
        this.notificationContext = this.notificationService.getNotificationContext();
        this.wsnWriter = WSNotificationWriter.getInstance();
        this.logger = logger;
        URL producerWsdlUrl;
        URL consumerWsdlUrl;
        try {
            producerWsdlUrl = new File(componentContext.getWorkspaceRoot() + File.separator
                    + Constants.Notification.NOTIFICATION_FOLDER + File.separator
                    + Constants.Notification.NOTIFICATION_WSDL_FOLDER + File.separator
                    + Constants.Notification.PRODUCER_WSDL).toURI().toURL();
            consumerWsdlUrl = new File(componentContext.getWorkspaceRoot() + File.separator
                    + Constants.Notification.NOTIFICATION_FOLDER + File.separator
                    + Constants.Notification.NOTIFICATION_WSDL_FOLDER + File.separator
                    + Constants.Notification.CONSUMER_WSDL).toURI().toURL();
        } catch (MalformedURLException e) {
            throw new WSDLException(e);
        }
        // initialize the notification producer
        this.notificationProducer = new ComponentNotificationProducer(
                componentContext.getDeliveryChannel(), logger, this, producerWsdlUrl);
        // initialize the notification consumer
        this.notificationConsumer = new ComponentNotificationConsumer(
                componentContext.getDeliveryChannel(), logger, consumerWsdlUrl);
        this.transformer = new ComponentTransformer(
                this.notificationProducer.getNotificationProducerSOAParameter(),
                this.notificationProducer.getSubscriptionManagerSOAParameter(),
                this.notificationConsumer.getNotificationConsumerSOAParameter());
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.ow2.petals.component.framework.api.notification.
     * NotificationBrokerController#getTransformer()
     */
    public final Transformer getTransformer() {
        return this.transformer;
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.ow2.petals.component.framework.api.notification.
     * NotificationBrokerController#destroyRegistration()
     */
    public final void destroyRegistration() throws WSNotificationException,
            WSNotificationExtensionException, WSAddressingException, MessagingException,
            PEtALSCDKException {
        this.logger.fine("Destroying the registration...");
        final DestroyRegistration destroyRegistration = WSNotificationFactory.getInstance()
                .createDestroyRegistration();
        if (notificationContext.getPublisherRegistrationReference() != null) {
            final ResourcesUuidType resourcesUuidType = WsnSpecificTypeHelper
                    .getResourcesUuidType(notificationContext.getPublisherRegistrationReference()
                            .getReferenceParameters());
            WsnSpecificTypeHelper.setResourcesUuidType(resourcesUuidType, destroyRegistration);
        } else {
            this.logger
                    .fine("No publisher registration so no uuid provided for destroy registration");
        }
        // FIXME : Unregister even if we are not registered???
        this.notificationService.getSender().send(destroyRegistration);
        try {
            this.notificationService.getNotificationStorageService().destroyPubRegResponseFile();
        } catch (Exception e) {
            throw new WSNotificationException(e);
        }
        this.logger.fine("Registration destroyed");
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.ow2.petals.component.framework.api.notification.
     * NotificationBrokerController#getCurrentMessage()
     */
    public final String getCurrentMessage() throws WSNotificationException, MessagingException,
            PEtALSCDKException, WSAddressingException {
        throw new NotImplementedException();
    }

    /*
     * (non-Javadoc)
     * 
     * @seeorg.ow2.petals.component.framework.api.notification.
     * NotificationBrokerController#notify(org.w3c.dom.Document,
     * java.lang.String,
     * com.ebmwebsourcing.wsstar.addressing.definition.api.EndpointReferenceType
     * )
     */
    public final void notify(final Document document, final String topic,
            final EndpointReferenceType subscriptionReference) throws WSNotificationException,
            WSAddressingException, WSDLException, WSNotificationExtensionException,
            MessagingException, PEtALSCDKException {
        SOAParameterType soaParameter = null;

        final Notify notify = WSNotificationFactory.getInstance().createNotify();
        final NotificationMessageHolderType notificationMessageHolderType = WSNotificationFactory
                .getInstance().createNotificationMessageHolderType();
        // Subscription Reference
        notificationMessageHolderType.setSubscriptionReference(subscriptionReference);
        // Topic
        final TopicExpressionType topicExpressionType = WSNotificationFactory.getInstance()
                .createTopicExpressionType();
        topicExpressionType.addTopicNameSpace("petals", Constants.WSStar.Topic.NAMESPACE_URI);
        topicExpressionType.setDialect(WstopConstants.CONCRETE_TOPIC_EXPRESSION_DIALECT_URI);
        topicExpressionType.setContent(topic);
        notificationMessageHolderType.setTopic(topicExpressionType);
        // Producer Reference
        final EndpointReferenceType producerReference = WSAddressingFactory.getInstance()
                .newEndpointReferenceType();
        final ReferenceParametersType referenceParametersType = producerReference
                .newReferenceParameters();
        soaParameter = this.notificationProducer.getNotificationProducerSOAParameter();
        WsnSpecificTypeHelper.setSOAParameter(soaParameter, referenceParametersType);
        producerReference.setReferenceParameters(referenceParametersType);
        notificationMessageHolderType.setProducerReference(producerReference);
        final Message msg = WSNotificationFactory.getInstance().createMessage();
        WsnSpecificTypeHelper.setContentToMessage(document.getDocumentElement(), msg);
        notificationMessageHolderType.setMessage(msg);
        notify.addNotificationMessage(notificationMessageHolderType);
        this.notificationService.getSender().send(notify,
                this.notificationContext.getConsumerReference());
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.ow2.petals.component.framework.api.notification.
     * NotificationBrokerController#registerPublisher()
     */
    public final void registerPublisher() throws WSNotificationException, MessagingException,
            PEtALSCDKException, WSAddressingException, WSNotificationExtensionException {
        this.logger.fine("Registering as publisher...");

        // Create register pub request
        RegisterPublisher registerPublisher = createRegisterPublisherRequest();

        // Create an async sender for register publisher request
        AsyncSender sender = this.notificationService.getAsyncSender(new StatusListener() {

            @Override
            public void onUnsubscribed(UnsubscribeResponse response) throws PEtALSCDKException {
            }

            @Override
            public void onUnregistered(DestroyRegistrationResponse response)
                    throws PEtALSCDKException {
            }

            @Override
            public void onSubscribed(SubscribeResponse response) throws PEtALSCDKException {
            }

            @Override
            public void onRegistered(RegisterPublisherResponse response) throws PEtALSCDKException {
                if (ComponentNotificationBrokerController.this.logger.isLoggable(Level.FINE)) {
                    ComponentNotificationBrokerController.this.logger
                            .fine("Processing registration response in response listener");
                }

                if (response != null) {
                    // persist response to be able to restore notif context on
                    // a component crash or when the component stops and can not
                    // unregister on broker
                    try {
                        ComponentNotificationBrokerController.this.notificationService
                                .getNotificationStorageService().store(response);
                    } catch (Exception e) {
                        throw new PEtALSCDKException(e);
                    }

                    // Update notification context with response information
                    ComponentNotificationBrokerController.this
                            .processRegisterPublisherResponse(response);

                    ComponentNotificationBrokerController.this.logger
                            .info("Successfuly register publisher on broker");
                } else {
                    throw new PEtALSCDKException("No response from the broker");
                }
            }

            @Override
            public void onException(Exception e) {
                final String msg = "Register publisher problem";
                ComponentNotificationBrokerController.this.logger.warning(msg);
                if (ComponentNotificationBrokerController.this.logger.isLoggable(Level.FINE)) {
                    ComponentNotificationBrokerController.this.logger.log(Level.FINE, msg, e);
                }
            }
        });

        // Send it asynchronously
        sender.send(registerPublisher);
    }

    public void processRegisterPublisherResponse(RegisterPublisherResponse response)
            throws PEtALSCDKException {
        // publisher registration reference : usefull to destroy
        // registration
        EndpointReferenceType publisherRegistrationReference = null;
        try {
            publisherRegistrationReference = response.getPublisherRegistrationReference();
        } catch (WSNotificationException e) {
            throw new PEtALSCDKException(e);
        }

        if (publisherRegistrationReference != null) {
            // In memory persistence for publisher registration ref
            this.notificationContext
                    .setPublisherRegistrationReference(publisherRegistrationReference);
            EndpointReferenceType consumerReference = null;
            try {
                consumerReference = response.getConsumerReference();
            } catch (WSNotificationException e) {
                // NOP
            }
            if (consumerReference != null) {
                // In memory persistence for notification consumer
                // ref
                this.notificationContext.setConsumerReference(consumerReference);
            }

            if (this.logger.isLoggable(Level.FINE)) {
                this.logger.fine("Publisher registered with reference "
                        + publisherRegistrationReference.getAddress());
            }
        } else {
            throw new PEtALSCDKException(
                    "The publisher reference MUST appear because the demand component has value \"true\"");
        }
    }

    private RegisterPublisher createRegisterPublisherRequest() throws WSNotificationException,
            WSAddressingException, MessagingException, WSNotificationExtensionException {
        RegisterPublisher registerPublisher = WSNotificationFactory.getInstance()
                .createRegisterPublisher();
        // Add publisher endpoint reference
        registerPublisher = this.transformer.addNotificationProducerSOAParameter(registerPublisher);
        // Add publisher supported topics to register publisher request
        createRegisterPublisherTopic(registerPublisher);
        // Demand = true : notification will be send only if there are
        // subscribers
        registerPublisher.setDemand(true);
        return registerPublisher;
    }

    private void createRegisterPublisherTopic(RegisterPublisher registerPublisher)
            throws WSNotificationException {
        final TopicExpressionType topic = WSNotificationFactory.getInstance()
                .createTopicExpressionType();

        // FIXME : This should be parameters...
        topic.addTopicNameSpace("petals", Constants.WSStar.Topic.NAMESPACE_URI);
        topic.setDialect(WstopConstants.XPATH_TOPIC_EXPRESSION_DIALECT_URI);
        topic.setContent("petals:component/cdk//*[@wstop:topic='true']");
        registerPublisher.addTopic(topic);
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.ow2.petals.component.framework.api.notification.
     * NotificationBrokerController #renew()
     */
    public void renew() {
        throw new NotImplementedException();
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.ow2.petals.component.framework.api.notification.
     * NotificationBrokerController#subscribe()
     */
    public void subscribe() throws WSNotificationException, MessagingException, PEtALSCDKException,
            WSAddressingException {
        throw new NotImplementedException();
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.ow2.petals.component.framework.api.notification.
     * NotificationBrokerController#unsubscribe()
     */
    public void unsubscribe() throws WSNotificationException, MessagingException,
            PEtALSCDKException, WSAddressingException {
        throw new NotImplementedException();
    }

    /*
     * (non-Javadoc)
     * 
     * @see
     * org.ow2.petals.component.framework.api.notification.NotificationActor
     * #isBrokeredNotification(javax.jbi.messaging.MessageExchange)
     */
    public final boolean isBrokeredNotification(final MessageExchange messageExchange) {
        if (messageExchange == null || messageExchange.getInterfaceName() == null
                || messageExchange.getInterfaceName().getNamespaceURI() == null) {
            return false;
        }
        return Constants.Notification.WS_BROKERED_NOTIFICATION_WSDL_NAMESPACE_URI
                .equals(messageExchange.getInterfaceName().getNamespaceURI());
    }

    /**
     * Get the notificationProducer
     * 
     * @return the notificationProducer
     */
    public final NotificationProducer getNotificationProducer() {
        return this.notificationProducer;
    }

    /**
     * Get the notificationConsumer
     * 
     * @return the notificationConsumer
     */
    public final NotificationConsumer getNotificationConsumer() {
        return this.notificationConsumer;
    }

    /*
     * (non-Javadoc)
     * 
     * @seeorg.ow2.petals.component.framework.api.notification.
     * NotificationBrokerController
     * #setNotificationBrokerReferenceAddress(java.lang.String)
     */
    public final void setNotificationBrokerReferenceAddress(
            final String notificationBrokerReferenceAddress) {
        this.transformer.setNotificationBrokerReferenceAddress(notificationBrokerReferenceAddress);
    }

    /*
     * (non-Javadoc)
     * 
     * @seeorg.ow2.petals.component.framework.api.notification.
     * NotificationBrokerController
     * #setPublisherRegistrationManagerReferenceAddress(java.lang.String)
     */
    public final void setPublisherRegistrationManagerReferenceAddress(
            final String publisherRegistrationManagerReferenceAddress) {
        this.transformer
                .setPublisherRegistrationManagerReferenceAddress(publisherRegistrationManagerReferenceAddress);
    }

    /*
     * (non-Javadoc)
     * 
     * @seeorg.ow2.petals.component.framework.api.notification.
     * NotificationBrokerController
     * #setSubscriptionManagerReferenceAddress(java.lang.String)
     */
    public final void setSubscriptionManagerReferenceAddress(
            final String subscriptionManagerReferenceAddress) {
        this.transformer
                .setSubscriptionManagerReferenceAddress(subscriptionManagerReferenceAddress);
    }

    /*
     * (non-Javadoc)
     * 
     * @seeorg.ow2.petals.component.framework.api.notification.
     * NotificationBrokerController#isInternalNotification(java.lang.String)
     */
    public final boolean isInternalNotification(final String address) {
        if (Constants.Notification.CDK_ADDRESS.equals(address)) {
            return true;
        }
        return false;
    }

    /**
     * Get the wsnWriter
     * 
     * @return the wsnWriter
     */
    public final WSNotificationWriter getWsnWriter() {
        return this.wsnWriter;
    }
}
