/**
 * PETALS: PETALS Services Platform Copyright (C) 2009 EBM WebSourcing
 * 
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or any later version.
 * 
 * This library is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA.
 * 
 * Initial developer(s): EBM WebSourcing
 */

package org.ow2.petals.component.framework.notification;

import com.ebmwebsourcing.wsstar.addressing.definition.api.EndpointReferenceType;
import com.ebmwebsourcing.wsstar.notification.definition.utils.WSNotificationException;

/**
 * Notification controller handles all references related to notifications at
 * the CDK level.
 * 
 * @author chamerling - eBM WebSourcing
 * 
 */
public class NotificationContext {

    static int MIN_DELAY = 2;

    static int MAX_DELAY = 120;

    static int DELAY_FACTOR = 2;

    /**
     * Means infinite retries...
     */
    static int NB_TRIES = 0;

    /**
     * The registration uuid for the cdk notifications
     */
    private String registrationUuid;

    /**
     * The consumer reference provided by the broker
     */
    private EndpointReferenceType consumerReference;

    /** The publisher registration reference provided by the broker. */
    private EndpointReferenceType publisherRegistrationReference;

    /* Retry values */

    private int minDelay = MIN_DELAY;

    private int maxDelay = MAX_DELAY;

    private int delayFactor = DELAY_FACTOR;

    private int nbRetry = NB_TRIES;

    /**
     * @throws WSNotificationException
     * 
     */
    public NotificationContext() {
    }

    /**
     * @return the registrationUuid
     */
    public String getRegistrationUuid() {
        return this.registrationUuid;
    }

    /**
     * @param registrationUuid
     *            the registrationUuid to set
     */
    public void setRegistrationUuid(String registrationUuid) {
        this.registrationUuid = registrationUuid;
    }

    public boolean isRegistered() {
        return this.publisherRegistrationReference != null;
    }

    public EndpointReferenceType getConsumerReference() {
        return this.consumerReference;
    }

    public void setConsumerReference(final EndpointReferenceType consumerReference) {
        this.consumerReference = consumerReference;
    }

    public int getMinDelay() {
        return this.minDelay;
    }

    public void setMinDelay(final int minDelay) {
        this.minDelay = minDelay;
    }

    public int getMaxDelay() {
        return this.maxDelay;
    }

    public void setMaxDelay(final int maxDelay) {
        this.maxDelay = maxDelay;
    }

    public int getDelayFactor() {
        return this.delayFactor;
    }

    public void setDelayFactor(final int delayFactor) {
        this.delayFactor = delayFactor;
    }

    public int getNbRetry() {
        return this.nbRetry;
    }

    public void setNbRetry(final int nbRetry) {
        this.nbRetry = nbRetry;
    }

    /**
     * @param publisherRegistrationReference
     */
    public void setPublisherRegistrationReference(
            final EndpointReferenceType publisherRegistrationReference) {
        this.publisherRegistrationReference = publisherRegistrationReference;
    }

    /**
     * @return the publisherRegistrationReference
     */
    public EndpointReferenceType getPublisherRegistrationReference() {
        return this.publisherRegistrationReference;
    }

}
