/**
 * PETALS - PETALS Services Platform. Copyright (c) 2009 EBM Websourcing,
 * http://www.ebmwebsourcing.com/
 * 
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version. This library is distributed in the hope that it will be
 * useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser
 * General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 * 
 */
package org.ow2.petals.component.framework.process.async;

import org.ow2.petals.component.framework.api.Constants.Component;
import org.ow2.petals.component.framework.api.message.Exchange;

/**
 * The Asynchronous context. <br/>
 * The Asynchronous context holds the required data to handle asynchronous
 * message responses in the JBI Listener. <br/>
 * The component developer can extends this class to provide its content of this
 * context.
 * 
 * @author rnaudin
 */
public class AsyncContext {

    private Exchange originalExchange;
    
    private long timeToLive = Component.DEFAULT_SEND_TIMEOUT;
    
    private long startTime;
    
    public AsyncContext(Exchange originalExchange) {
        this.originalExchange = originalExchange;
        
    }
    
    public AsyncContext(Exchange originalExchange, Long timeToLive) {
        this(originalExchange);
        this.timeToLive = timeToLive;
    }

    public Exchange getOriginalExchange() {
        return this.originalExchange;
    }
    
    public long getTimeToLive() {
        return this.timeToLive;
    }
    
    public void setStartTime(long startTime) {
        this.startTime = startTime; 
    }
    
    /**
     * Return the expiration time in milliseconds.
     * 
     * @return
     */
    public long getExpirationTime() {
        if (timeToLive != 0 ) {
        return startTime + timeToLive;
        } else {
            return 0;
        }
    }
}
