
package org.ow2.petals.component.framework.util;

import java.util.concurrent.ThreadFactory;
import java.util.concurrent.atomic.AtomicInteger;

/**
 * A customizable implementation of a {@link ThreadFactory}.<br/>
 * Customizable properties are :<br/>
 * - Thread name prefix <br/>
 * - Thread priority <br/>
 * - Daemon nature
 * 
 * @author ofabre
 * 
 */
public class CustomizableThreadFactory implements ThreadFactory {

    private String threadNamePrefix;

    private int threadPriority = Thread.NORM_PRIORITY;

    private boolean daemon = false;

    final AtomicInteger threadNumber = new AtomicInteger(1);

    /**
     * Create a new {@link ThreadFactory} with a specified thread name prefix
     * 
     * @param threadNamePrefix
     *            the thread name prefix. Will be followed by the thread number.
     */
    public CustomizableThreadFactory(String threadNamePrefix) {
        super();
        this.threadNamePrefix = threadNamePrefix;
    }

    /**
     * Create a new {@link ThreadFactory} with specified thread name prefix,
     * thread priority and daemon nature
     * 
     * @param threadNamePrefix
     *            the thread name prefix. Will be followed by the thread number.
     * @param threadPriority
     *            the threads priority
     * @param daemon
     *            the daemon nature of created threads
     */
    public CustomizableThreadFactory(String threadNamePrefix, int threadPriority, boolean daemon) {
        super();
        this.threadNamePrefix = threadNamePrefix;
        this.threadPriority = threadPriority;
        this.daemon = daemon;
    }

    public void setThreadNamePrefix(String threadNamePrefix) {
        this.threadNamePrefix = threadNamePrefix;
    }

    public void setThreadPriority(int threadPriority) {
        this.threadPriority = threadPriority;
    }

    public void setDaemon(boolean daemon) {
        this.daemon = daemon;
    }

    @Override
    public Thread newThread(Runnable r) {
        StringBuilder name = new StringBuilder(threadNamePrefix);
        name.append(threadNumber.getAndIncrement());

        Thread t = new Thread(r, name.toString());

        t.setDaemon(daemon);
        t.setPriority(threadPriority);
        return t;
    }

}
